from __future__ import annotations

from dataclasses import dataclass, field
from datetime import datetime

from mrp.models import DiscoveredAgent, Message


@dataclass
class Contact:
    public_key: str
    display_name: str | None
    capabilities: list[str]
    alias: str | None = None
    last_seen: datetime | None = None


class SessionState:
    """In-memory state for the MCP session."""

    def __init__(self):
        self.contacts: dict[str, Contact] = {}
        self.aliases: dict[str, str] = {}
        self.threads: dict[str, str] = {}
        self.unread: list[Message] = []
        self.last_poll_cursor: str | None = None

    def add_contact(self, agent: DiscoveredAgent, alias: str | None = None) -> Contact:
        """Add or update a contact. Auto-index by display_name."""
        contact = Contact(
            public_key=agent.public_key,
            display_name=agent.display_name,
            capabilities=agent.capabilities,
            alias=alias,
            last_seen=agent.last_active_at,
        )
        self.contacts[agent.public_key] = contact
        if agent.display_name:
            self.aliases[agent.display_name.lower()] = agent.public_key
        if alias:
            self.aliases[alias.lower()] = agent.public_key
        return contact

    def resolve_recipient(self, identifier: str) -> str:
        """Resolve a name, alias, or public key to a public key."""
        # Direct public key lookup
        if identifier in self.contacts:
            return identifier
        # Alias or display name lookup (case-insensitive)
        lower = identifier.lower()
        if lower in self.aliases:
            return self.aliases[lower]
        # Fuzzy: check if identifier is a substring of any display name
        for pk, contact in self.contacts.items():
            if contact.display_name and lower in contact.display_name.lower():
                return pk
        raise ValueError(
            f"Unknown recipient: {identifier}. Use mrp_discover to find agents first."
        )



from __future__ import annotations

import asyncio
import json

from mcp.server.fastmcp import FastMCP
from mrp import Agent
from mrp.models import DiscoveredAgent

from mrp_mcp.formatting import format_message
from mrp_mcp.state import SessionState


def register(mcp: FastMCP, agent: Agent, state: SessionState) -> None:
    @mcp.tool()
    async def mrp_send(
        to: str,
        message: str,
        thread_id: str | None = None,
    ) -> str:
        """Send a message to another agent on the MRP network.

        Args:
            to: The recipient - can be an agent's display name, alias, or public key.
                Use mrp_discover first to find agents and add them to contacts.
            message: The message text to send.
            thread_id: Optional thread ID to organize this into a conversation.
                Use a descriptive ID like "translation-task-1".

        Returns:
            Confirmation with the message ID and delivery status.
        """
        recipient_pk = state.resolve_recipient(to)
        body = {"text": message}

        result = await asyncio.to_thread(
            agent.send, to=recipient_pk, body=body, thread=thread_id
        )

        contact = state.contacts.get(recipient_pk)
        name = contact.display_name if contact else recipient_pk[:12] + "..."
        return (
            f"Message sent to **{name}**.\n"
            f"Message ID: {result.message_id}\n"
            f"Thread: {thread_id or '(none)'}\n"
            f"Expires: {result.expires_at.isoformat()}"
        )

    @mcp.tool()
    async def mrp_reply(
        message_id: str,
        message: str,
    ) -> str:
        """Reply to a specific message you received.

        This automatically sets the recipient to the original sender and
        links the reply to the original message's conversation thread.

        Args:
            message_id: The ID of the message to reply to (from mrp_check_messages).
            message: The reply text.

        Returns:
            Confirmation with the reply's message ID.
        """
        original = await asyncio.to_thread(agent.client.get_message, message_id)
        body = {"text": message}
        result = await asyncio.to_thread(agent.reply, original, body)

        sender = state.contacts.get(original.sender_key)
        name = sender.display_name if sender else original.sender_key[:12] + "..."
        return (
            f"Reply sent to **{name}**.\n"
            f"Message ID: {result.message_id}\n"
            f"In reply to: {message_id}"
        )

    @mcp.tool()
    async def mrp_check_messages(
        from_agent: str | None = None,
        limit: int = 10,
    ) -> str:
        """Check for new incoming messages.

        Call this to see if any agents have sent you messages. Messages are
        marked as delivered once you read them.

        Args:
            from_agent: Optional filter - only show messages from this agent
                (name, alias, or public key).
            limit: Maximum number of messages to return (1-50, default 10).

        Returns:
            A list of new messages with sender, content, and message IDs
            (needed for mrp_reply).
        """
        sender_key = None
        if from_agent:
            sender_key = state.resolve_recipient(from_agent)

        # Check background poller's unread queue first
        messages = []
        if state.unread:
            for msg in list(state.unread):
                if sender_key and msg.sender_key != sender_key:
                    continue
                messages.append(msg)
                state.unread.remove(msg)
                if len(messages) >= limit:
                    break

        # If we need more, poll the relay
        if len(messages) < limit:
            result = await asyncio.to_thread(
                agent.client.poll_messages,
                status="sent",
                limit=limit - len(messages),
                sender_key=sender_key,
            )
            messages.extend(result.messages)

        # Add senders to contacts
        for msg in messages:
            if msg.sender_key not in state.contacts:
                try:
                    info = await asyncio.to_thread(
                        agent.client.get_agent, msg.sender_key
                    )
                    state.add_contact(
                        DiscoveredAgent(
                            public_key=info.public_key,
                            display_name=info.display_name,
                            last_active_at=info.last_active_at,
                            capabilities=info.capabilities,
                        )
                    )
                except Exception:
                    pass

        if not messages:
            return "No new messages."

        lines = [f"You have {len(messages)} new message(s):\n"]
        for msg in messages:
            lines.append("---")
            lines.append(format_message(msg, state))
            lines.append("")

        return "\n".join(lines)



from __future__ import annotations

from datetime import datetime, timezone
from unittest.mock import MagicMock, PropertyMock

import pytest

from mrp.models import (
    AgentInfo,
    Blob,
    DiscoveredAgent,
    Message,
    MessageStatus,
    PollResult,
    SendResult,
)
from mrp_mcp.state import SessionState


def _now():
    return datetime.now(timezone.utc)


@pytest.fixture
def state():
    return SessionState()


@pytest.fixture
def mock_client():
    client = MagicMock()

    client.get_agent.return_value = AgentInfo(
        public_key="test-pk-1234567890abcdefghijklmnopqrstuv",
        display_name="Test Agent",
        status="active",
        metadata={},
        capabilities=["text:chat"],
        created_at=_now(),
        last_active_at=_now(),
    )

    client.update_agent.return_value = AgentInfo(
        public_key="test-pk-1234567890abcdefghijklmnopqrstuv",
        display_name="Updated Agent",
        status="active",
        metadata={},
        capabilities=["text:chat", "text:summarize"],
        created_at=_now(),
        last_active_at=_now(),
    )

    client.poll_messages.return_value = PollResult(messages=[], next_cursor=None)

    client.send_message.return_value = SendResult(
        message_id="msg_001",
        status="sent",
        created_at=_now(),
        expires_at=_now(),
    )

    client.get_message.return_value = Message(
        message_id="msg_001",
        sender_key="sender-pk-abcdefghijklmnopqrstuvwxyz1234",
        recipient_key="test-pk-1234567890abcdefghijklmnopqrstuv",
        content_type="application/json",
        body={"text": "Hello!"},
        attachments=[],
        status="sent",
        thread_id=None,
        in_reply_to=None,
        metadata={},
        created_at=_now(),
        expires_at=_now(),
    )

    client.get_message_status.return_value = MessageStatus(
        status="delivered",
        created_at=_now(),
        delivered_at=_now(),
        expires_at=_now(),
    )

    client.get_thread.return_value = PollResult(messages=[], next_cursor=None)

    client.upload_blob.return_value = Blob(
        blob_id="blob_001",
        hash="abc123",
        size=1024,
        content_type="application/octet-stream",
        created_at=_now(),
        expires_at=_now(),
    )

    client.download_blob.return_value = (b"file-content", "application/octet-stream")

    return client


@pytest.fixture
def mock_agent(mock_client):
    agent = MagicMock()
    agent.client = mock_client
    type(agent).public_key = PropertyMock(
        return_value="test-pk-1234567890abcdefghijklmnopqrstuv"
    )

    _discover_results = [
        DiscoveredAgent(
            public_key="agent-pk-abcdefghijklmnopqrstuvwxyz12345",
            display_name="TranslatorBot",
            last_active_at=_now(),
            capabilities=["text:translate"],
        ),
    ]
    agent.discover.return_value = _discover_results
    agent.discover_search.return_value = _discover_results

    agent.send.return_value = SendResult(
        message_id="msg_002",
        status="sent",
        created_at=_now(),
        expires_at=_now(),
    )

    agent.reply.return_value = SendResult(
        message_id="msg_003",
        status="sent",
        created_at=_now(),
        expires_at=_now(),
    )

    agent.upload.return_value = Blob(
        blob_id="blob_001",
        hash="abc123",
        size=1024,
        content_type="application/octet-stream",
        created_at=_now(),
        expires_at=_now(),
    )

    agent.download.return_value = (b"file-content", "application/octet-stream")

    return agent


@pytest.fixture
def sample_agents():
    return [
        DiscoveredAgent(
            public_key="pk-agent-a-abcdefghijklmnopqrstuvwx",
            display_name="Agent Alpha",
            last_active_at=_now(),
            capabilities=["text:chat", "text:summarize"],
        ),
        DiscoveredAgent(
            public_key="pk-agent-b-abcdefghijklmnopqrstuvwx",
            display_name="Agent Beta",
            last_active_at=_now(),
            capabilities=["code:review"],
        ),
    ]


@pytest.fixture
def sample_messages():
    return [
        Message(
            message_id="msg_100",
            sender_key="sender-pk-abcdefghijklmnopqrstuvwxyz1234",
            recipient_key="test-pk-1234567890abcdefghijklmnopqrstuv",
            content_type="application/json",
            body={"text": "Hello there!"},
            attachments=[],
            status="sent",
            thread_id="thread-1",
            in_reply_to=None,
            metadata={},
            created_at=_now(),
            expires_at=_now(),
        ),
        Message(
            message_id="msg_101",
            sender_key="sender-pk-abcdefghijklmnopqrstuvwxyz1234",
            recipient_key="test-pk-1234567890abcdefghijklmnopqrstuv",
            content_type="application/json",
            body={"text": "How are you?"},
            attachments=[],
            status="sent",
            thread_id="thread-1",
            in_reply_to="msg_100",
            metadata={},
            created_at=_now(),
            expires_at=_now(),
        ),
    ]



from __future__ import annotations

import asyncio

from mcp.server.fastmcp import FastMCP
from mrp import Agent

from mrp_mcp.state import SessionState


def register(mcp: FastMCP, agent: Agent, state: SessionState) -> None:
    @mcp.tool()
    async def mrp_upload_blob(
        file_path: str,
        content_type: str = "application/octet-stream",
    ) -> str:
        """Upload a file to share with other agents.

        The file is stored on the relay and you get a blob ID that you can
        include when sending messages (via structured_body with attachments).

        Args:
            file_path: Path to the file to upload. Max 100 MiB.
            content_type: MIME type of the file (e.g. "image/png", "application/pdf").

        Returns:
            The blob ID and metadata. Use this blob_id in mrp_send's structured_body.
        """

        def _upload():
            with open(file_path, "rb") as f:
                data = f.read()
            return agent.upload(data, content_type=content_type)

        blob = await asyncio.to_thread(_upload)

        return (
            f"File uploaded successfully.\n"
            f"Blob ID: {blob.blob_id}\n"
            f"Size: {blob.size} bytes\n"
            f"Hash: {blob.hash}\n"
            f"Expires: {blob.expires_at.isoformat()}\n\n"
            f"To send this file, include the blob_id as an attachment when sending a message."
        )

    @mcp.tool()
    async def mrp_download_blob(
        blob_id: str,
        save_to: str,
    ) -> str:
        """Download a file (blob) from the relay.

        Use this to download files attached to messages you've received.

        Args:
            blob_id: The blob ID from a message's attachments.
            save_to: Local file path to save the downloaded file.

        Returns:
            Confirmation with file path and size.
        """

        def _download():
            data, ct = agent.download(blob_id)
            with open(save_to, "wb") as f:
                f.write(data)
            return data, ct

        data, content_type_result = await asyncio.to_thread(_download)

        return (
            f"File downloaded successfully.\n"
            f"Saved to: {save_to}\n"
            f"Size: {len(data)} bytes\n"
            f"Content type: {content_type_result}"
        )



"""Tests for MRP MCP server module."""

from __future__ import annotations

import asyncio
from datetime import datetime, timezone
from unittest.mock import AsyncMock, MagicMock, patch, PropertyMock

import pytest

from mrp.models import Message, PollResult
from mrp_mcp.config import Config
from mrp_mcp.server import background_poller, create_server
from mrp_mcp.state import SessionState


def _now():
    return datetime.now(timezone.utc)


def _make_message(msg_id="msg_001"):
    return Message(
        message_id=msg_id,
        sender_key="sender-pk-abcdefghijklmnopqrstuvwxyz1234",
        recipient_key="test-pk-1234567890abcdefghijklmnopqrstuv",
        content_type="application/json",
        body={"text": "Hello!"},
        attachments=[],
        status="sent",
        thread_id=None,
        in_reply_to=None,
        metadata={},
        created_at=_now(),
        expires_at=_now(),
    )


class TestBackgroundPoller:
    @pytest.mark.asyncio
    async def test_background_poller_no_messages(self):
        agent = MagicMock()
        agent.client.poll_messages.return_value = PollResult(messages=[], next_cursor=None)
        state = SessionState()

        with patch("mrp_mcp.server.asyncio.sleep", new_callable=AsyncMock) as mock_sleep:
            mock_sleep.side_effect = [None, asyncio.CancelledError()]
            try:
                await background_poller(agent, state, 5.0)
            except asyncio.CancelledError:
                pass

        assert state.unread == []
        assert state.last_poll_cursor is None

    @pytest.mark.asyncio
    async def test_background_poller_with_messages(self):
        messages = [_make_message("msg_001"), _make_message("msg_002")]
        agent = MagicMock()
        agent.client.poll_messages.return_value = PollResult(
            messages=messages, next_cursor="cursor-abc"
        )
        state = SessionState()

        with patch("mrp_mcp.server.asyncio.sleep", new_callable=AsyncMock) as mock_sleep:
            mock_sleep.side_effect = [None, asyncio.CancelledError()]
            with patch("mrp_mcp.server.asyncio.to_thread", new_callable=AsyncMock) as mock_to_thread:
                mock_to_thread.return_value = PollResult(
                    messages=messages, next_cursor="cursor-abc"
                )
                try:
                    await background_poller(agent, state, 5.0)
                except asyncio.CancelledError:
                    pass

        assert len(state.unread) == 2
        assert state.last_poll_cursor == "cursor-abc"

    @pytest.mark.asyncio
    async def test_background_poller_updates_cursor(self):
        agent = MagicMock()
        state = SessionState()
        state.last_poll_cursor = "old-cursor"

        with patch("mrp_mcp.server.asyncio.sleep", new_callable=AsyncMock) as mock_sleep:
            mock_sleep.side_effect = [None, asyncio.CancelledError()]
            with patch("mrp_mcp.server.asyncio.to_thread", new_callable=AsyncMock) as mock_to_thread:
                mock_to_thread.return_value = PollResult(
                    messages=[_make_message()], next_cursor="new-cursor"
                )
                try:
                    await background_poller(agent, state, 5.0)
                except asyncio.CancelledError:
                    pass

        assert state.last_poll_cursor == "new-cursor"

    @pytest.mark.asyncio
    async def test_background_poller_exception_handling(self):
        agent = MagicMock()
        state = SessionState()

        with patch("mrp_mcp.server.asyncio.sleep", new_callable=AsyncMock) as mock_sleep:
            mock_sleep.side_effect = [None, asyncio.CancelledError()]
            with patch("mrp_mcp.server.asyncio.to_thread", new_callable=AsyncMock) as mock_to_thread:
                mock_to_thread.side_effect = RuntimeError("Network error")
                try:
                    await background_poller(agent, state, 5.0)
                except asyncio.CancelledError:
                    pass

        # Should silently handle exception and continue
        assert state.unread == []
        assert state.last_poll_cursor is None


class TestCreateServer:
    @patch("mrp_mcp.server.agent_tools")
    @patch("mrp_mcp.server.blobs")
    @patch("mrp_mcp.server.threads")
    @patch("mrp_mcp.server.messaging")
    @patch("mrp_mcp.server.discovery")
    @patch("mrp_mcp.server.Agent")
    def test_create_server_auto_register(
        self, mock_agent_cls, mock_disc, mock_msg, mock_thr, mock_blobs, mock_at
    ):
        mock_agent = MagicMock()
        mock_agent_cls.return_value = mock_agent

        config = Config(
            relay_url="http://localhost:8080",
            key_file="/tmp/test.key",
            agent_name="TestBot",
            auto_register=True,
        )
        mcp = create_server(config)

        mock_agent.register.assert_called_once()
        assert mcp is not None

    @patch("mrp_mcp.server.agent_tools")
    @patch("mrp_mcp.server.blobs")
    @patch("mrp_mcp.server.threads")
    @patch("mrp_mcp.server.messaging")
    @patch("mrp_mcp.server.discovery")
    @patch("mrp_mcp.server.Agent")
    def test_create_server_no_register(
        self, mock_agent_cls, mock_disc, mock_msg, mock_thr, mock_blobs, mock_at
    ):
        mock_agent = MagicMock()
        mock_agent_cls.return_value = mock_agent

        config = Config(
            relay_url="http://localhost:8080",
            auto_register=False,
        )
        mcp = create_server(config)

        mock_agent.register.assert_not_called()
        assert mcp is not None



from __future__ import annotations

import json
import os
from argparse import Namespace
from pathlib import Path

import pytest

from mrp_mcp.config import Config, load_config


class TestConfig:
    def test_defaults(self):
        config = Config()
        assert config.relay_url == ""
        assert config.key_file is None
        assert config.agent_name is None
        assert config.capabilities == []
        assert config.metadata == {}
        assert config.poll_interval == 0
        assert config.auto_register is True

    def test_load_from_env(self, monkeypatch):
        monkeypatch.setenv("MRP_RELAY_URL", "https://relay.example.com")
        monkeypatch.setenv("MRP_KEY_FILE", "./test.key")
        monkeypatch.setenv("MRP_AGENT_NAME", "TestBot")
        monkeypatch.setenv("MRP_CAPABILITIES", "text:chat,text:summarize")
        monkeypatch.setenv("MRP_METADATA", '{"v": "1.0"}')
        monkeypatch.setenv("MRP_POLL_INTERVAL", "5")

        config = load_config()
        assert config.relay_url == "https://relay.example.com"
        assert config.key_file == "./test.key"
        assert config.agent_name == "TestBot"
        assert config.capabilities == ["text:chat", "text:summarize"]
        assert config.metadata == {"v": "1.0"}
        assert config.poll_interval == 5

    def test_load_from_cli_args(self, monkeypatch):
        monkeypatch.delenv("MRP_RELAY_URL", raising=False)
        args = Namespace(
            relay="https://cli-relay.com",
            key="./cli.key",
            name="CLI Bot",
            cap=["code:review"],
            poll_interval=10,
            config=None,
        )
        config = load_config(args)
        assert config.relay_url == "https://cli-relay.com"
        assert config.key_file == "./cli.key"
        assert config.agent_name == "CLI Bot"
        assert config.capabilities == ["code:review"]
        assert config.poll_interval == 10

    def test_cli_overrides_env(self, monkeypatch):
        monkeypatch.setenv("MRP_RELAY_URL", "https://env-relay.com")
        monkeypatch.setenv("MRP_AGENT_NAME", "EnvBot")

        args = Namespace(
            relay="https://cli-relay.com",
            key=None,
            name=None,
            cap=None,
            poll_interval=0,
            config=None,
        )
        config = load_config(args)
        assert config.relay_url == "https://cli-relay.com"
        assert config.agent_name == "EnvBot"  # not overridden since CLI name is None

    def test_load_from_config_file(self, tmp_path, monkeypatch):
        monkeypatch.delenv("MRP_RELAY_URL", raising=False)
        config_data = {
            "relay": "https://file-relay.com",
            "key_file": "./file.key",
            "name": "FileBot",
            "capabilities": ["text:chat"],
            "poll_interval": 3,
        }
        config_file = tmp_path / "mrp-mcp.json"
        config_file.write_text(json.dumps(config_data))

        args = Namespace(
            relay=None, key=None, name=None, cap=None,
            poll_interval=0, config=str(config_file),
        )
        config = load_config(args)
        assert config.relay_url == "https://file-relay.com"
        assert config.key_file == "./file.key"
        assert config.agent_name == "FileBot"

    def test_missing_relay_url_raises(self, monkeypatch):
        monkeypatch.delenv("MRP_RELAY_URL", raising=False)
        with pytest.raises(ValueError, match="relay URL is required"):
            load_config()

    def test_env_overrides_config_file(self, tmp_path, monkeypatch):
        config_data = {"relay": "https://file-relay.com", "name": "FileBot"}
        config_file = tmp_path / "mrp-mcp.json"
        config_file.write_text(json.dumps(config_data))

        monkeypatch.setenv("MRP_RELAY_URL", "https://env-relay.com")

        args = Namespace(
            relay=None, key=None, name=None, cap=None,
            poll_interval=0, config=str(config_file),
        )
        config = load_config(args)
        assert config.relay_url == "https://env-relay.com"
        assert config.agent_name == "FileBot"  # from config file



from __future__ import annotations

from datetime import datetime, timezone

import pytest

from mrp.models import DiscoveredAgent
from mrp_mcp.state import SessionState


def _now():
    return datetime.now(timezone.utc)


def _agent(pk="pk-abc", name="TestBot", caps=None):
    return DiscoveredAgent(
        public_key=pk,
        display_name=name,
        last_active_at=_now(),
        capabilities=caps or ["text:chat"],
    )


class TestSessionState:
    def test_add_contact(self, state):
        agent = _agent()
        contact = state.add_contact(agent)
        assert contact.public_key == "pk-abc"
        assert contact.display_name == "TestBot"
        assert "pk-abc" in state.contacts

    def test_add_contact_indexes_by_display_name(self, state):
        state.add_contact(_agent(pk="pk-1", name="Alpha Bot"))
        assert "alpha bot" in state.aliases
        assert state.aliases["alpha bot"] == "pk-1"

    def test_add_contact_with_alias(self, state):
        state.add_contact(_agent(pk="pk-1", name="Bot"), alias="mybot")
        assert "mybot" in state.aliases
        assert state.aliases["mybot"] == "pk-1"

    def test_resolve_by_public_key(self, state):
        state.add_contact(_agent(pk="pk-123"))
        assert state.resolve_recipient("pk-123") == "pk-123"

    def test_resolve_by_display_name(self, state):
        state.add_contact(_agent(pk="pk-123", name="TranslatorBot"))
        assert state.resolve_recipient("TranslatorBot") == "pk-123"

    def test_resolve_case_insensitive(self, state):
        state.add_contact(_agent(pk="pk-123", name="TranslatorBot"))
        assert state.resolve_recipient("translatorbot") == "pk-123"
        assert state.resolve_recipient("TRANSLATORBOT") == "pk-123"

    def test_resolve_by_alias(self, state):
        state.add_contact(_agent(pk="pk-123", name="Bot"), alias="translator")
        assert state.resolve_recipient("translator") == "pk-123"

    def test_resolve_by_substring(self, state):
        state.add_contact(_agent(pk="pk-123", name="TranslatorBot"))
        assert state.resolve_recipient("Translator") == "pk-123"

    def test_resolve_unknown_raises(self, state):
        with pytest.raises(ValueError, match="Unknown recipient"):
            state.resolve_recipient("nobody")

    def test_update_existing_contact(self, state):
        state.add_contact(_agent(pk="pk-1", name="OldName"))
        state.add_contact(_agent(pk="pk-1", name="NewName"))
        assert state.contacts["pk-1"].display_name == "NewName"
        assert "newname" in state.aliases

    def test_add_contact_without_display_name(self, state):
        agent = _agent(pk="pk-1", name=None)
        contact = state.add_contact(agent)
        assert contact.display_name is None
        # Should still be findable by public key
        assert state.resolve_recipient("pk-1") == "pk-1"

    def test_multiple_contacts(self, state):
        state.add_contact(_agent(pk="pk-a", name="Alpha"))
        state.add_contact(_agent(pk="pk-b", name="Beta"))
        assert len(state.contacts) == 2
        assert state.resolve_recipient("Alpha") == "pk-a"
        assert state.resolve_recipient("Beta") == "pk-b"

    def test_unread_queue(self, state):
        assert state.unread == []
        assert state.last_poll_cursor is None



from __future__ import annotations

from datetime import datetime, timezone
from unittest.mock import MagicMock

import pytest

from mcp.server.fastmcp import FastMCP
from mrp.models import DiscoveredAgent, Message, PollResult, SendResult

from mrp_mcp.tools.messaging import register


def _now():
    return datetime.now(timezone.utc)


@pytest.fixture
def mcp_server(mock_agent, state):
    mcp = FastMCP("test")
    register(mcp, mock_agent, state)
    return mcp


@pytest.fixture
def state_with_contact(state):
    state.add_contact(
        DiscoveredAgent(
            public_key="agent-pk-abcdefghijklmnopqrstuvwxyz12345",
            display_name="TranslatorBot",
            last_active_at=_now(),
            capabilities=["text:translate"],
        )
    )
    return state


@pytest.mark.asyncio
async def test_send_by_name(mcp_server, mock_agent, state_with_contact):
    result = await mcp_server.call_tool(
        "mrp_send", {"to": "TranslatorBot", "message": "Hello!"}
    )
    text = result[0][0].text

    mock_agent.send.assert_called_once()
    call_kwargs = mock_agent.send.call_args
    assert call_kwargs.kwargs["to"] == "agent-pk-abcdefghijklmnopqrstuvwxyz12345"
    assert call_kwargs.kwargs["body"] == {"text": "Hello!"}
    assert "Message sent to **TranslatorBot**" in text
    assert "msg_002" in text


@pytest.mark.asyncio
async def test_send_with_thread(mcp_server, mock_agent, state_with_contact):
    result = await mcp_server.call_tool(
        "mrp_send",
        {"to": "TranslatorBot", "message": "Hi", "thread_id": "thread-1"},
    )
    text = result[0][0].text

    call_kwargs = mock_agent.send.call_args
    assert call_kwargs.kwargs["thread"] == "thread-1"
    assert "thread-1" in text


@pytest.mark.asyncio
async def test_send_unknown_recipient(mcp_server):
    with pytest.raises(Exception, match="Unknown recipient"):
        await mcp_server.call_tool(
            "mrp_send", {"to": "nobody", "message": "Hello!"}
        )


@pytest.mark.asyncio
async def test_reply(mcp_server, mock_agent, state_with_contact):
    # Set up the original message's sender to be in contacts
    state_with_contact.add_contact(
        DiscoveredAgent(
            public_key="sender-pk-abcdefghijklmnopqrstuvwxyz1234",
            display_name="SenderBot",
            last_active_at=_now(),
            capabilities=[],
        )
    )

    result = await mcp_server.call_tool(
        "mrp_reply", {"message_id": "msg_001", "message": "Got it!"}
    )
    text = result[0][0].text

    mock_agent.client.get_message.assert_called_once_with("msg_001")
    mock_agent.reply.assert_called_once()
    assert "Reply sent to **SenderBot**" in text
    assert "msg_003" in text


@pytest.mark.asyncio
async def test_check_messages_empty(mcp_server, mock_agent):
    result = await mcp_server.call_tool("mrp_check_messages", {})
    text = result[0][0].text
    assert "No new messages" in text


@pytest.mark.asyncio
async def test_check_messages_with_results(mcp_server, mock_agent, state, sample_messages):
    mock_agent.client.poll_messages.return_value = PollResult(
        messages=sample_messages, next_cursor=None
    )
    # Mock get_agent for unknown senders
    mock_agent.client.get_agent.return_value = MagicMock(
        public_key="sender-pk-abcdefghijklmnopqrstuvwxyz1234",
        display_name="SenderBot",
        last_active_at=_now(),
        capabilities=[],
    )

    result = await mcp_server.call_tool("mrp_check_messages", {})
    text = result[0][0].text

    assert "2 new message(s)" in text
    assert "Hello there!" in text
    assert "How are you?" in text


@pytest.mark.asyncio
async def test_check_messages_from_unread_queue(mcp_server, mock_agent, state, sample_messages):
    # Add messages to unread queue (simulating background poller)
    state.unread.extend(sample_messages)
    mock_agent.client.get_agent.return_value = MagicMock(
        public_key="sender-pk-abcdefghijklmnopqrstuvwxyz1234",
        display_name="SenderBot",
        last_active_at=_now(),
        capabilities=[],
    )

    result = await mcp_server.call_tool("mrp_check_messages", {"limit": 1})
    text = result[0][0].text

    assert "1 new message(s)" in text
    # One message should remain in unread
    assert len(state.unread) == 1



from __future__ import annotations

import pytest

from mcp.server.fastmcp import FastMCP

from mrp_mcp.tools.blobs import register


@pytest.fixture
def mcp_server(mock_agent, state):
    mcp = FastMCP("test")
    register(mcp, mock_agent, state)
    return mcp


@pytest.mark.asyncio
async def test_upload_blob(mcp_server, mock_agent, tmp_path):
    test_file = tmp_path / "test.txt"
    test_file.write_text("hello world")

    result = await mcp_server.call_tool(
        "mrp_upload_blob",
        {"file_path": str(test_file), "content_type": "text/plain"},
    )
    text = result[0][0].text

    mock_agent.upload.assert_called_once()
    assert "blob_001" in text
    assert "1024 bytes" in text
    assert "File uploaded successfully" in text


@pytest.mark.asyncio
async def test_download_blob(mcp_server, mock_agent, tmp_path):
    save_path = tmp_path / "downloaded.bin"

    result = await mcp_server.call_tool(
        "mrp_download_blob",
        {"blob_id": "blob_001", "save_to": str(save_path)},
    )
    text = result[0][0].text

    mock_agent.download.assert_called_once_with("blob_001")
    assert "File downloaded successfully" in text
    assert str(save_path) in text
    assert save_path.read_bytes() == b"file-content"



import argparse
import sys

from mrp_mcp.config import load_config
from mrp_mcp.server import run_server


def main():
    parser = argparse.ArgumentParser(description="MRP MCP Tool Server")
    parser.add_argument("--relay", help="MRP relay URL")
    parser.add_argument("--key", help="Path to Ed25519 key file")
    parser.add_argument("--name", help="Agent display name")
    parser.add_argument(
        "--cap", action="append", help="Agent capability (repeatable)"
    )
    parser.add_argument(
        "--poll-interval",
        type=int,
        default=0,
        dest="poll_interval",
        help="Background poll interval in seconds (0=disabled)",
    )
    parser.add_argument("--config", help="Path to config file")
    parser.add_argument(
        "--ephemeral",
        action="store_true",
        default=False,
        help="Use an ephemeral in-memory key (do not persist identity)",
    )
    args = parser.parse_args()

    try:
        config = load_config(args)
    except ValueError as e:
        print(f"Error: {e}", file=sys.stderr)
        sys.exit(1)

    run_server(config)


if __name__ == "__main__":
    main()



from __future__ import annotations

import asyncio

from mcp.server.fastmcp import FastMCP
from mrp import Agent

from mrp_mcp.formatting import format_thread_message
from mrp_mcp.state import SessionState


def register(mcp: FastMCP, agent: Agent, state: SessionState) -> None:
    @mcp.tool()
    async def mrp_read_thread(thread_id: str, limit: int = 20) -> str:
        """Read a full conversation thread.

        Shows all messages in a thread in chronological order.

        Args:
            thread_id: The thread ID to read.
            limit: Maximum messages to return (1-100, default 20).

        Returns:
            The conversation history in chronological order.
        """
        result = await asyncio.to_thread(
            agent.client.get_thread, thread_id, limit=limit
        )

        if not result.messages:
            return f"No messages found in thread '{thread_id}'."

        lines = [f"Thread: {thread_id} ({len(result.messages)} messages)\n"]
        for msg in result.messages:
            lines.append(format_thread_message(msg, state, agent.public_key))
            lines.append("")

        if result.next_cursor:
            lines.append("(More messages available. Call again with a higher limit.)")

        return "\n".join(lines)

    @mcp.tool()
    async def mrp_message_status(message_id: str) -> str:
        """Check the delivery status of a message you sent.

        Args:
            message_id: The message ID returned by mrp_send or mrp_reply.

        Returns:
            The message status: sent (not yet read), delivered (read), or expired.
        """
        status = await asyncio.to_thread(
            agent.client.get_message_status, message_id
        )

        lines = [
            f"Message: {message_id}",
            f"Status: **{status.status}**",
            f"Created: {status.created_at.isoformat()}",
        ]
        if status.delivered_at:
            lines.append(f"Delivered: {status.delivered_at.isoformat()}")
        lines.append(f"Expires: {status.expires_at.isoformat()}")

        return "\n".join(lines)



"""Tests for MRP MCP server CLI entry point."""

from __future__ import annotations

import argparse
from unittest.mock import MagicMock, patch

import pytest

from mrp_mcp.__main__ import main


class TestMain:
    @patch("mrp_mcp.__main__.run_server")
    @patch("mrp_mcp.__main__.load_config")
    def test_argument_parsing(self, mock_load_config, mock_run_server, monkeypatch):
        mock_config = MagicMock()
        mock_load_config.return_value = mock_config

        monkeypatch.setattr(
            "sys.argv",
            [
                "mrp-mcp",
                "--relay", "http://localhost:9090",
                "--key", "/tmp/my.key",
                "--name", "MyBot",
                "--poll-interval", "10",
            ],
        )
        main()

        mock_load_config.assert_called_once()
        args = mock_load_config.call_args[0][0]
        assert args.relay == "http://localhost:9090"
        assert args.key == "/tmp/my.key"
        assert args.name == "MyBot"
        assert args.poll_interval == 10
        mock_run_server.assert_called_once_with(mock_config)

    @patch("mrp_mcp.__main__.run_server")
    @patch("mrp_mcp.__main__.load_config")
    def test_repeatable_cap(self, mock_load_config, mock_run_server, monkeypatch):
        mock_load_config.return_value = MagicMock()

        monkeypatch.setattr(
            "sys.argv",
            ["mrp-mcp", "--relay", "http://localhost:8080", "--cap", "text:chat", "--cap", "text:translate"],
        )
        main()

        args = mock_load_config.call_args[0][0]
        assert args.cap == ["text:chat", "text:translate"]

    @patch("mrp_mcp.__main__.load_config")
    def test_config_error_stderr(self, mock_load_config, monkeypatch, capsys):
        mock_load_config.side_effect = ValueError("Missing relay URL")

        monkeypatch.setattr("sys.argv", ["mrp-mcp"])

        with pytest.raises(SystemExit) as exc_info:
            main()

        assert exc_info.value.code == 1
        captured = capsys.readouterr()
        assert "Error: Missing relay URL" in captured.err

    @patch("mrp_mcp.__main__.run_server")
    @patch("mrp_mcp.__main__.load_config")
    def test_run_server_called(self, mock_load_config, mock_run_server, monkeypatch):
        mock_config = MagicMock()
        mock_load_config.return_value = mock_config

        monkeypatch.setattr("sys.argv", ["mrp-mcp", "--relay", "http://localhost:8080"])
        main()

        mock_run_server.assert_called_once_with(mock_config)



from __future__ import annotations

import asyncio

from mcp.server.fastmcp import FastMCP
from mrp import Agent

from mrp_mcp.state import SessionState


def register(mcp: FastMCP, agent: Agent, state: SessionState) -> None:
    @mcp.tool()
    async def mrp_discover(
        capability: str | None = None,
        capability_prefix: str | None = None,
        name: str | None = None,
    ) -> str:
        """Find agents on the MRP network by capability, prefix, or name.

        Use this to discover agents that can help with a task. At least one
        search parameter is required.

        Search modes (can be combined):
        - capability: exact match (e.g. "text:translate")
        - capability_prefix: namespace match (e.g. "text:" finds all text agents)
        - name: substring match on agent display names

        Capabilities use colon-separated namespaces like:
        - text:translate, text:summarize, text:chat
        - code:review, code:generate, code:explain
        - image:generate, image:analyze

        Args:
            capability: Exact capability to search for (e.g. "text:translate")
            capability_prefix: Capability prefix to match (e.g. "text:")
            name: Agent display name substring to search for

        Returns:
            A list of agents with their names, capabilities, and when they were last active.
            These agents are saved to your contacts for easy reference in mrp_send.
        """
        if not capability and not capability_prefix and not name:
            return "Error: at least one of capability, capability_prefix, or name is required."

        caps = [capability] if capability else None
        results = await asyncio.to_thread(
            agent.discover_search,
            capabilities=caps,
            capability_prefix=capability_prefix,
            name=name,
        )

        for a in results:
            state.add_contact(a)

        if not results:
            parts = []
            if capability:
                parts.append(f"capability '{capability}'")
            if capability_prefix:
                parts.append(f"prefix '{capability_prefix}'")
            if name:
                parts.append(f"name '{name}'")
            return f"No agents found matching {', '.join(parts)}."

        lines = [f"Found {len(results)} agent(s):\n"]
        for i, a in enumerate(results, 1):
            display = a.display_name or "(unnamed)"
            lines.append(f"{i}. **{display}**")
            lines.append(f"   Public key: {a.public_key}")
            lines.append(f"   Capabilities: {', '.join(a.capabilities)}")
            lines.append(f"   Last active: {a.last_active_at.isoformat()}")
            lines.append("")

        lines.append("These agents are now in your contacts. Use their name in mrp_send.")
        return "\n".join(lines)

    @mcp.tool()
    async def mrp_list_contacts() -> str:
        """List all agents you've discovered in this session.

        Shows agents found via previous mrp_discover calls, along with
        their capabilities and any aliases you've set.

        Returns:
            A list of known agents with their names and capabilities.
        """
        if not state.contacts:
            return "No contacts yet. Use mrp_discover to find agents."

        lines = ["Known contacts:\n"]
        for pk, contact in state.contacts.items():
            display = contact.display_name or "(unnamed)"
            alias = f" (alias: {contact.alias})" if contact.alias else ""
            caps = ", ".join(contact.capabilities) if contact.capabilities else "none"
            lines.append(f"- **{display}**{alias}: {caps}")
        return "\n".join(lines)

    @mcp.tool()
    async def mrp_capabilities() -> str:
        """List all capabilities available on the MRP network.

        Use this to browse what capabilities exist before discovering specific agents.
        Shows each capability tag and how many agents offer it.

        Returns:
            A list of capability tags with agent counts, sorted by popularity.
        """
        result = await asyncio.to_thread(agent.list_capabilities)

        if not result.capabilities:
            return "No capabilities registered on the network."

        lines = [f"Found {len(result.capabilities)} capability tag(s):\n"]
        for c in result.capabilities:
            lines.append(f"- **{c.tag}** ({c.agent_count} agent{'s' if c.agent_count != 1 else ''})")
        return "\n".join(lines)



from __future__ import annotations

import pytest

from mcp.server.fastmcp import FastMCP
from mrp.models import PollResult

from mrp_mcp.tools.threads import register


@pytest.fixture
def mcp_server(mock_agent, state):
    mcp = FastMCP("test")
    register(mcp, mock_agent, state)
    return mcp


@pytest.mark.asyncio
async def test_read_thread_empty(mcp_server, mock_agent):
    result = await mcp_server.call_tool("mrp_read_thread", {"thread_id": "thread-1"})
    text = result[0][0].text
    assert "No messages found" in text


@pytest.mark.asyncio
async def test_read_thread_with_messages(mcp_server, mock_agent, sample_messages):
    mock_agent.client.get_thread.return_value = PollResult(
        messages=sample_messages, next_cursor=None
    )

    result = await mcp_server.call_tool("mrp_read_thread", {"thread_id": "thread-1"})
    text = result[0][0].text

    assert "thread-1" in text
    assert "2 messages" in text
    assert "Hello there!" in text
    assert "How are you?" in text


@pytest.mark.asyncio
async def test_read_thread_with_pagination(mcp_server, mock_agent, sample_messages):
    mock_agent.client.get_thread.return_value = PollResult(
        messages=sample_messages, next_cursor="cursor-abc"
    )

    result = await mcp_server.call_tool("mrp_read_thread", {"thread_id": "thread-1"})
    text = result[0][0].text

    assert "More messages available" in text


@pytest.mark.asyncio
async def test_message_status(mcp_server, mock_agent):
    result = await mcp_server.call_tool(
        "mrp_message_status", {"message_id": "msg_001"}
    )
    text = result[0][0].text

    mock_agent.client.get_message_status.assert_called_once_with("msg_001")
    assert "msg_001" in text
    assert "delivered" in text.lower()



from __future__ import annotations

import json
from datetime import datetime

from mrp.models import Message

from mrp_mcp.state import SessionState


def format_message(msg: Message, state: SessionState, own_key: str | None = None) -> str:
    """Format a single message for display."""
    sender = state.contacts.get(msg.sender_key)
    if own_key and msg.sender_key == own_key:
        name = "You"
    elif sender and sender.display_name:
        name = sender.display_name
    else:
        name = msg.sender_key[:12] + "..."

    lines = []
    lines.append(f"**From:** {name}")
    lines.append(f"**Message ID:** {msg.message_id}")
    if msg.thread_id:
        lines.append(f"**Thread:** {msg.thread_id}")
    lines.append(f"**Time:** {msg.created_at.isoformat()}")
    lines.append("**Content:**")
    if isinstance(msg.body, dict) and "text" in msg.body:
        lines.append(msg.body["text"])
    else:
        lines.append(f"```json\n{json.dumps(msg.body, indent=2)}\n```")
    if msg.attachments:
        lines.append(f"**Attachments:** {len(msg.attachments)} file(s)")
        for att in msg.attachments:
            size = f", {att.size} bytes" if att.size else ""
            ct = f" ({att.content_type}{size})" if att.content_type else ""
            lines.append(f"  - {att.blob_id}{ct}")
    return "\n".join(lines)


def format_thread_message(msg: Message, state: SessionState, own_key: str) -> str:
    """Format a message within a thread view."""
    sender = state.contacts.get(msg.sender_key)
    if msg.sender_key == own_key:
        label = "You"
    elif sender and sender.display_name:
        label = sender.display_name
    else:
        label = msg.sender_key[:12] + "..."

    time_str = msg.created_at.strftime("%H:%M:%S")
    lines = [f"**[{label}]** ({time_str})"]
    if isinstance(msg.body, dict) and "text" in msg.body:
        lines.append(f"  {msg.body['text']}")
    else:
        lines.append(f"  ```json\n  {json.dumps(msg.body, indent=2)}\n  ```")
    return "\n".join(lines)


def format_contact_name(state: SessionState, public_key: str) -> str:
    """Get a display name for a public key."""
    contact = state.contacts.get(public_key)
    if contact and contact.display_name:
        return contact.display_name
    return public_key[:12] + "..."



from __future__ import annotations

import pytest

from mcp.server.fastmcp import FastMCP

from mrp_mcp.tools.agent import register


@pytest.fixture
def mcp_server(mock_agent, state):
    mcp = FastMCP("test")
    register(mcp, mock_agent, state)
    return mcp


@pytest.mark.asyncio
async def test_whoami(mcp_server, mock_agent):
    result = await mcp_server.call_tool("mrp_whoami", {})
    text = result[0][0].text

    mock_agent.client.get_agent.assert_called_once_with(mock_agent.public_key)
    assert "Identity" in text
    assert "Test Agent" in text
    assert "text:chat" in text
    assert "active" in text


@pytest.mark.asyncio
async def test_update_profile_name(mcp_server, mock_agent):
    result = await mcp_server.call_tool(
        "mrp_update_profile", {"name": "New Name"}
    )
    text = result[0][0].text

    mock_agent.client.update_agent.assert_called_once()
    call_kwargs = mock_agent.client.update_agent.call_args
    assert call_kwargs.kwargs["display_name"] == "New Name"
    assert "Profile updated" in text


@pytest.mark.asyncio
async def test_update_profile_capabilities(mcp_server, mock_agent):
    result = await mcp_server.call_tool(
        "mrp_update_profile", {"capabilities": "text:chat,code:review"}
    )
    text = result[0][0].text

    call_kwargs = mock_agent.client.update_agent.call_args
    assert call_kwargs.kwargs["capabilities"] == ["text:chat", "code:review"]
    assert "Profile updated" in text


@pytest.mark.asyncio
async def test_update_profile_metadata(mcp_server, mock_agent):
    result = await mcp_server.call_tool(
        "mrp_update_profile", {"metadata": {"version": "2.0"}}
    )
    text = result[0][0].text

    call_kwargs = mock_agent.client.update_agent.call_args
    assert call_kwargs.kwargs["metadata"] == {"version": "2.0"}
    assert "Profile updated" in text


@pytest.mark.asyncio
async def test_register_webhook(mcp_server, mock_agent):
    result = await mcp_server.call_tool(
        "mrp_register_webhook",
        {"url": "https://example.com/webhook", "secret": "s3cret"},
    )
    text = result[0][0].text

    mock_agent.client._request.assert_called_once()
    assert "Webhook registered" in text
    assert "https://example.com/webhook" in text



from __future__ import annotations

import asyncio
import json

from mcp.server.fastmcp import FastMCP
from mrp import Agent

from mrp_mcp.state import SessionState


def register(mcp: FastMCP, agent: Agent, state: SessionState) -> None:
    @mcp.tool()
    async def mrp_whoami() -> str:
        """Show this agent's identity and profile on the MRP network.

        Returns your public key, display name, capabilities, and status.
        """
        info = await asyncio.to_thread(agent.client.get_agent, agent.public_key)

        lines = [
            "**Identity**",
            f"Public key: {info.public_key}",
            f"Display name: {info.display_name or '(not set)'}",
            f"Status: {info.status}",
            f"Capabilities: {', '.join(info.capabilities) if info.capabilities else '(none)'}",
            f"Created: {info.created_at.isoformat()}",
            f"Last active: {info.last_active_at.isoformat()}",
        ]
        if info.metadata:
            lines.append(f"Metadata: {json.dumps(info.metadata)}")

        return "\n".join(lines)

    @mcp.tool()
    async def mrp_update_profile(
        name: str | None = None,
        capabilities: str | None = None,
        metadata: dict | None = None,
    ) -> str:
        """Update this agent's profile on the MRP network.

        Other agents see your profile when they discover you.

        Args:
            name: Display name for this agent.
            capabilities: Comma-separated list of capabilities
                (e.g. "text:chat,text:summarize"). Max 20.
            metadata: Key-value metadata pairs. Max 16 keys, 256 chars each.

        Returns:
            The updated profile.
        """
        caps_list = None
        if capabilities:
            caps_list = [c.strip() for c in capabilities.split(",")]

        info = await asyncio.to_thread(
            agent.client.update_agent,
            agent.public_key,
            display_name=name,
            capabilities=caps_list,
            metadata=metadata,
        )
        caps_str = ", ".join(info.capabilities) if info.capabilities else "(none)"
        return f"Profile updated.\nName: {info.display_name}\nCapabilities: {caps_str}"

    @mcp.tool()
    async def mrp_register_webhook(url: str, secret: str) -> str:
        """Register a webhook URL for push message delivery.

        Instead of polling for messages, the relay will POST messages
        to this URL as they arrive.

        Args:
            url: The publicly reachable HTTPS URL to receive messages.
            secret: A shared secret for HMAC-SHA256 signature verification.

        Returns:
            Confirmation of webhook registration.
        """

        def _register():
            return agent.client._request(
                "PUT",
                f"/v1/agents/{agent.public_key}/webhook",
                body={"url": url, "secret": secret},
            )

        await asyncio.to_thread(_register)
        return f"Webhook registered.\nURL: {url}\nEnabled: true"



from __future__ import annotations

import pytest

from mcp.server.fastmcp import FastMCP
from mrp.models import CapabilitiesResult, CapabilityInfo

from mrp_mcp.tools.discovery import register


@pytest.fixture
def mcp_server(mock_agent, state):
    mcp = FastMCP("test")
    register(mcp, mock_agent, state)
    return mcp


@pytest.mark.asyncio
async def test_discover_finds_agents(mcp_server, mock_agent, state):
    result = await mcp_server.call_tool("mrp_discover", {"capability": "text:translate"})
    text = result[0][0].text

    mock_agent.discover_search.assert_called_once_with(
        capabilities=["text:translate"],
        capability_prefix=None,
        name=None,
    )
    assert "TranslatorBot" in text
    assert "Found 1 agent(s)" in text
    assert "text:translate" in text
    # Agent should be added to contacts
    assert len(state.contacts) == 1
    assert "translatorbot" in state.aliases


@pytest.mark.asyncio
async def test_discover_by_prefix(mcp_server, mock_agent, state):
    result = await mcp_server.call_tool("mrp_discover", {"capability_prefix": "text:"})
    text = result[0][0].text

    mock_agent.discover_search.assert_called_once_with(
        capabilities=None,
        capability_prefix="text:",
        name=None,
    )
    assert "Found 1 agent(s)" in text


@pytest.mark.asyncio
async def test_discover_by_name(mcp_server, mock_agent, state):
    result = await mcp_server.call_tool("mrp_discover", {"name": "translator"})
    text = result[0][0].text

    mock_agent.discover_search.assert_called_once_with(
        capabilities=None,
        capability_prefix=None,
        name="translator",
    )
    assert "Found 1 agent(s)" in text


@pytest.mark.asyncio
async def test_discover_no_params(mcp_server, mock_agent):
    result = await mcp_server.call_tool("mrp_discover", {})
    text = result[0][0].text

    assert "Error" in text
    mock_agent.discover_search.assert_not_called()


@pytest.mark.asyncio
async def test_discover_no_results(mcp_server, mock_agent):
    mock_agent.discover_search.return_value = []
    result = await mcp_server.call_tool("mrp_discover", {"capability": "nonexistent"})
    text = result[0][0].text

    assert "No agents found" in text


@pytest.mark.asyncio
async def test_list_contacts_empty(mcp_server):
    result = await mcp_server.call_tool("mrp_list_contacts", {})
    text = result[0][0].text

    assert "No contacts yet" in text


@pytest.mark.asyncio
async def test_list_contacts_with_agents(mcp_server, state, sample_agents):
    for agent in sample_agents:
        state.add_contact(agent)

    result = await mcp_server.call_tool("mrp_list_contacts", {})
    text = result[0][0].text

    assert "Agent Alpha" in text
    assert "Agent Beta" in text
    assert "text:chat" in text
    assert "code:review" in text


@pytest.mark.asyncio
async def test_capabilities(mcp_server, mock_agent):
    mock_agent.list_capabilities.return_value = CapabilitiesResult(
        capabilities=[
            CapabilityInfo(tag="text:translate", agent_count=5),
            CapabilityInfo(tag="code:review", agent_count=2),
        ]
    )

    result = await mcp_server.call_tool("mrp_capabilities", {})
    text = result[0][0].text

    assert "text:translate" in text
    assert "5 agents" in text
    assert "code:review" in text
    assert "2 agents" in text


@pytest.mark.asyncio
async def test_capabilities_empty(mcp_server, mock_agent):
    mock_agent.list_capabilities.return_value = CapabilitiesResult(capabilities=[])

    result = await mcp_server.call_tool("mrp_capabilities", {})
    text = result[0][0].text

    assert "No capabilities" in text



from __future__ import annotations

import asyncio
import logging
from collections.abc import AsyncIterator
from contextlib import asynccontextmanager

from mcp.server.fastmcp import FastMCP
from mrp import Agent

from mrp_mcp.config import Config
from mrp_mcp.state import SessionState
from mrp_mcp.tools import agent as agent_tools
from mrp_mcp.tools import blobs, discovery, messaging, threads

logger = logging.getLogger("mrp_mcp")


async def background_poller(
    mrp_agent: Agent,
    state: SessionState,
    interval: float,
) -> None:
    """Poll for messages in the background."""
    while True:
        await asyncio.sleep(interval)
        try:
            result = await asyncio.to_thread(
                mrp_agent.client.poll_messages,
                status="sent",
                limit=50,
                cursor=state.last_poll_cursor,
            )
            if result.messages:
                state.unread.extend(result.messages)
            if result.next_cursor:
                state.last_poll_cursor = result.next_cursor
        except Exception:
            pass


def create_server(config: Config) -> FastMCP:
    """Create and configure the MCP server with all tools registered."""
    mrp_agent = Agent(
        relay=config.relay_url,
        key_file=config.key_file,
        name=config.agent_name,
        capabilities=config.capabilities or None,
        metadata=config.metadata or None,
    )

    if config.auto_register:
        mrp_agent.register()

    state = SessionState()

    # Set up lifespan for background poller
    @asynccontextmanager
    async def lifespan(server: FastMCP) -> AsyncIterator[None]:
        poller_task = None
        if config.poll_interval > 0:
            poller_task = asyncio.create_task(
                background_poller(mrp_agent, state, config.poll_interval)
            )
        try:
            yield
        finally:
            if poller_task:
                poller_task.cancel()

    mcp = FastMCP("mrp-mcp", lifespan=lifespan)

    discovery.register(mcp, mrp_agent, state)
    messaging.register(mcp, mrp_agent, state)
    threads.register(mcp, mrp_agent, state)
    blobs.register(mcp, mrp_agent, state)
    agent_tools.register(mcp, mrp_agent, state)

    return mcp


def run_server(config: Config) -> None:
    """Create and run the MCP server (blocking)."""
    mcp = create_server(config)
    mcp.run()



from __future__ import annotations

import json
import os
from dataclasses import dataclass, field
from pathlib import Path

from mrp.auth import default_key_path


@dataclass
class Config:
    relay_url: str = ""
    key_file: str | None = None
    agent_name: str | None = None
    capabilities: list[str] = field(default_factory=list)
    metadata: dict[str, str] = field(default_factory=dict)
    poll_interval: int = 0
    auto_register: bool = True
    ephemeral: bool = False


def load_config(args=None) -> Config:
    """Load config with priority: CLI args > env vars > config file > defaults."""
    config = Config()

    # 1. Config file (lowest priority)
    config_path = None
    if args and getattr(args, "config", None):
        config_path = args.config
    elif os.environ.get("MRP_CONFIG"):
        config_path = os.environ["MRP_CONFIG"]
    else:
        default_path = Path("mrp-mcp.json")
        if default_path.exists():
            config_path = str(default_path)

    if config_path:
        with open(config_path) as f:
            data = json.load(f)
        config.relay_url = data.get("relay", config.relay_url)
        config.key_file = data.get("key_file", config.key_file)
        config.agent_name = data.get("name", config.agent_name)
        config.capabilities = data.get("capabilities", config.capabilities)
        config.metadata = data.get("metadata", config.metadata)
        config.poll_interval = data.get("poll_interval", config.poll_interval)
        config.auto_register = data.get("auto_register", config.auto_register)

    # 2. Environment variables
    if os.environ.get("MRP_RELAY_URL"):
        config.relay_url = os.environ["MRP_RELAY_URL"]
    if os.environ.get("MRP_KEY_FILE"):
        config.key_file = os.environ["MRP_KEY_FILE"]
    if os.environ.get("MRP_AGENT_NAME"):
        config.agent_name = os.environ["MRP_AGENT_NAME"]
    if os.environ.get("MRP_CAPABILITIES"):
        config.capabilities = os.environ["MRP_CAPABILITIES"].split(",")
    if os.environ.get("MRP_METADATA"):
        config.metadata = json.loads(os.environ["MRP_METADATA"])
    if os.environ.get("MRP_POLL_INTERVAL"):
        config.poll_interval = int(os.environ["MRP_POLL_INTERVAL"])

    # 3. CLI args (highest priority)
    if args:
        if getattr(args, "relay", None):
            config.relay_url = args.relay
        if getattr(args, "key", None):
            config.key_file = args.key
        if getattr(args, "name", None):
            config.agent_name = args.name
        if getattr(args, "cap", None):
            config.capabilities = args.cap
        if getattr(args, "poll_interval", None) is not None:
            config.poll_interval = args.poll_interval
        if getattr(args, "ephemeral", False):
            config.ephemeral = True

    if not config.relay_url:
        raise ValueError(
            "MRP relay URL is required. Set MRP_RELAY_URL env var, "
            "pass --relay, or add 'relay' to mrp-mcp.json."
        )

    # Default to persistent identity unless --ephemeral or an explicit key was set
    if config.key_file is None and not config.ephemeral:
        config.key_file = default_key_path(config.agent_name)

    return config



"""Tests for MRP MCP server formatting utilities."""

from __future__ import annotations

from datetime import datetime, timezone

import pytest

from mrp.models import Attachment, DiscoveredAgent, Message
from mrp_mcp.formatting import format_contact_name, format_message, format_thread_message
from mrp_mcp.state import SessionState


def _now():
    return datetime.now(timezone.utc)


def _make_message(
    sender_key="sender-pk-abcdefghijklmnopqrstuvwxyz1234",
    body=None,
    thread_id=None,
    attachments=None,
    message_id="msg_100",
):
    return Message(
        message_id=message_id,
        sender_key=sender_key,
        recipient_key="test-pk-1234567890abcdefghijklmnopqrstuv",
        content_type="application/json",
        body=body if body is not None else {"text": "Hello there!"},
        attachments=attachments or [],
        status="sent",
        thread_id=thread_id,
        in_reply_to=None,
        metadata={},
        created_at=_now(),
        expires_at=_now(),
    )


def _state_with_contact(pk="sender-pk-abcdefghijklmnopqrstuvwxyz1234", name="Alice"):
    state = SessionState()
    state.add_contact(
        DiscoveredAgent(
            public_key=pk,
            display_name=name,
            last_active_at=_now(),
            capabilities=["text:chat"],
        )
    )
    return state


class TestFormatMessage:
    def test_format_message_known_contact(self, state):
        state.add_contact(
            DiscoveredAgent(
                public_key="sender-pk-abcdefghijklmnopqrstuvwxyz1234",
                display_name="Alice",
                last_active_at=_now(),
                capabilities=[],
            )
        )
        msg = _make_message()
        result = format_message(msg, state)
        assert "**From:** Alice" in result

    def test_format_message_unknown_contact(self, state):
        msg = _make_message(sender_key="unknown-pk-abcdefghijklmnopqrstuvwxyz")
        result = format_message(msg, state)
        assert "**From:** unknown-pk-a..." in result

    def test_format_message_from_self(self, state):
        own_key = "my-pk-abcdefghijklmnopqrstuvwxyz12345"
        msg = _make_message(sender_key=own_key)
        result = format_message(msg, state, own_key=own_key)
        assert "**From:** You" in result

    def test_format_message_text_body(self, state):
        msg = _make_message(body={"text": "Hello world"})
        result = format_message(msg, state)
        assert "Hello world" in result
        assert "```json" not in result

    def test_format_message_json_body(self, state):
        msg = _make_message(body={"action": "ping", "data": 42})
        result = format_message(msg, state)
        assert "```json" in result
        assert '"action"' in result
        assert '"ping"' in result

    def test_format_message_with_thread(self, state):
        msg = _make_message(thread_id="thread-abc")
        result = format_message(msg, state)
        assert "**Thread:** thread-abc" in result

    def test_format_message_with_attachments(self, state):
        atts = [
            Attachment(blob_id="blob_001", content_type="image/png", size=2048),
            Attachment(blob_id="blob_002", content_type="text/plain", size=None),
        ]
        msg = _make_message(attachments=atts)
        result = format_message(msg, state)
        assert "**Attachments:** 2 file(s)" in result
        assert "blob_001" in result
        assert "image/png" in result
        assert "2048 bytes" in result
        assert "blob_002" in result

    def test_format_message_no_attachments(self, state):
        msg = _make_message(attachments=[])
        result = format_message(msg, state)
        assert "Attachments" not in result

    def test_format_message_contains_message_id(self, state):
        msg = _make_message(message_id="msg_xyz")
        result = format_message(msg, state)
        assert "**Message ID:** msg_xyz" in result

    def test_format_message_contains_time(self, state):
        msg = _make_message()
        result = format_message(msg, state)
        assert "**Time:**" in result


class TestFormatThreadMessage:
    def test_format_thread_message_self(self, state):
        own_key = "my-pk-abcdefghijklmnopqrstuvwxyz12345"
        msg = _make_message(sender_key=own_key, body={"text": "My reply"})
        result = format_thread_message(msg, state, own_key)
        assert "**[You]**" in result
        assert "My reply" in result

    def test_format_thread_message_contact(self, state):
        state.add_contact(
            DiscoveredAgent(
                public_key="sender-pk-abcdefghijklmnopqrstuvwxyz1234",
                display_name="Bob",
                last_active_at=_now(),
                capabilities=[],
            )
        )
        msg = _make_message(body={"text": "Hi from Bob"})
        result = format_thread_message(msg, state, "my-pk")
        assert "**[Bob]**" in result
        assert "Hi from Bob" in result

    def test_format_thread_message_unknown(self, state):
        msg = _make_message(
            sender_key="unknown-pk-abcdefghijklmnopqrstuvwxyz",
            body={"text": "Mystery"},
        )
        result = format_thread_message(msg, state, "my-pk")
        assert "**[unknown-pk-a...]**" in result

    def test_format_thread_message_indent(self, state):
        msg = _make_message(body={"text": "Indented text"})
        result = format_thread_message(msg, state, "my-pk")
        # Text body lines should be indented with two spaces
        assert "  Indented text" in result

    def test_format_thread_message_json_body(self, state):
        msg = _make_message(body={"action": "update"})
        result = format_thread_message(msg, state, "my-pk")
        assert "```json" in result


class TestFormatContactName:
    def test_format_contact_name_known(self, state):
        state.add_contact(
            DiscoveredAgent(
                public_key="pk-known-abcdefghijklmnopqrstuvwxy",
                display_name="Charlie",
                last_active_at=_now(),
                capabilities=[],
            )
        )
        result = format_contact_name(state, "pk-known-abcdefghijklmnopqrstuvwxy")
        assert result == "Charlie"

    def test_format_contact_name_no_name(self, state):
        state.add_contact(
            DiscoveredAgent(
                public_key="pk-noname-abcdefghijklmnopqrstuvw",
                display_name=None,
                last_active_at=_now(),
                capabilities=[],
            )
        )
        result = format_contact_name(state, "pk-noname-abcdefghijklmnopqrstuvw")
        assert result == "pk-noname-ab..."

    def test_format_contact_name_unknown(self, state):
        result = format_contact_name(state, "pk-unknown-abcdefghijklmnopqrstu")
        assert result == "pk-unknown-a..."
